/*
 * OperationAbstraction.hpp
 *
 *  Created on: 10. 7. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _OPERATION_ABSTRACTION_HPP_
#define _OPERATION_ABSTRACTION_HPP_

#include <alib/memory>
#include <alib/string>
#include <alib/typeindex>
#include <alib/set>

#include <common/ParamQualifiers.hpp>

namespace abstraction {

class OperationAbstraction : public std::enable_shared_from_this < OperationAbstraction > {
protected:
	virtual bool run ( ) = 0;

public:
	virtual bool attachInput ( const std::shared_ptr < OperationAbstraction > & input, unsigned index, bool move, bool checkInput ) = 0;
	virtual bool detachInput ( unsigned index ) = 0;

	virtual ~OperationAbstraction ( ) noexcept {
	}

	virtual bool inputsAttached ( ) const = 0;
	virtual bool eval ( ) = 0;
	virtual unsigned numberOfParams ( ) const = 0;
	virtual bool cached ( ) const = 0;
	virtual void reset ( ) = 0;

	virtual ext::type_index getParamTypeIndex ( unsigned index ) const = 0;
	virtual ext::type_index getReturnTypeIndex ( ) const = 0;

	std::string getParamType ( unsigned index ) const {
		return ext::to_string ( getParamTypeIndex ( index ) );
	}

	virtual ext::set < abstraction::ParamQualifiers::ParamQualifier > getParamTypeQualifiers ( unsigned index ) const = 0;

	std::string getReturnType ( ) const {
		return ext::to_string ( getReturnTypeIndex ( ) );
	}

	virtual ext::set < abstraction::ParamQualifiers::ParamQualifier > getReturnTypeQualifiers ( ) const = 0;

	virtual std::shared_ptr < abstraction::OperationAbstraction > getProxyAbstraction ( ) {
		return shared_from_this();
	}
};

} /* namespace abstraction */

#endif /* _OPERATION_ABSTRACTION_HPP_ */
