/*
 * Rename.cpp
 *
 *  Created on: Dec 9, 2013
 *	Author: Jan Travnicek
 */

#include "Rename.h"
#include <registration/AlgoRegistration.hpp>

namespace automaton {

namespace simplify {

auto RenameDFA = registration::AbstractRegister < Rename, automaton::DFA < DefaultSymbolType, unsigned >, const automaton::DFA < > & > ( Rename::rename, "automaton" ).setDocumentation (
"Rename automaton's states.\n\
\n\
@param automaton finite automaton to normalize\n\
@return @p automaton with renamed properties" );

auto RenameNFA = registration::AbstractRegister < Rename, automaton::NFA < DefaultSymbolType, unsigned >, const automaton::NFA < > & > ( Rename::rename, "automaton" ).setDocumentation (
"Rename automaton's states.\n\
\n\
@param automaton finite automaton to normalize\n\
@return @p automaton with renamed properties" );

auto RenameDPDA = registration::AbstractRegister < Rename, automaton::DPDA < DefaultSymbolType, DefaultEpsilonType, unsigned, unsigned >, const automaton::DPDA < > & > ( Rename::rename, "automaton" ).setDocumentation (
"Rename automaton's states and pushdown store symbols.\n\
\n\
@param pda pushdown automaton to rename\n\
@return @p pda with renamed states and pushdown store symbols" );

auto RenameNPDA = registration::AbstractRegister < Rename, automaton::NPDA < DefaultSymbolType, DefaultEpsilonType, unsigned, unsigned >, const automaton::NPDA < > & > ( Rename::rename, "automaton" ).setDocumentation (
"Rename automaton's states and pushdown store symbols.\n\
\n\
@param pda pushdown automaton to rename\n\
@return @p pda with renamed states and pushdown store symbols" );

auto RenameSinglePopDPDA = registration::AbstractRegister < Rename, automaton::SinglePopDPDA < DefaultSymbolType, DefaultEpsilonType, unsigned, unsigned >, const automaton::SinglePopDPDA < > & > ( Rename::rename, "automaton" ).setDocumentation (
"Rename automaton's states and pushdown store symbols.\n\
\n\
@param pda pushdown automaton to rename\n\
@return @p pda with renamed states and pushdown store symbols" );

auto RenameInputDrivenDPDA = registration::AbstractRegister < Rename, automaton::InputDrivenDPDA < DefaultSymbolType, unsigned, unsigned >, const automaton::InputDrivenDPDA < > & > ( Rename::rename, "automaton" ).setDocumentation (
"Rename automaton's states and pushdown store symbols.\n\
\n\
@param pda pushdown automaton to rename\n\
@return @p pda with renamed states and pushdown store symbols" );

auto RenameVisiblyPushdownDPDA = registration::AbstractRegister < Rename, automaton::VisiblyPushdownDPDA < DefaultSymbolType, unsigned, unsigned >, const automaton::VisiblyPushdownDPDA < > & > ( Rename::rename, "automaton" ).setDocumentation (
"Rename automaton's states and pushdown store symbols.\n\
\n\
@param pda pushdown automaton to rename\n\
@return @p pda with renamed states and pushdown store symbols" );

auto RenameRealTimeHeightDeterministicDPDA = registration::AbstractRegister < Rename, automaton::RealTimeHeightDeterministicDPDA < DefaultSymbolType, DefaultEpsilonType, unsigned, unsigned >, const automaton::RealTimeHeightDeterministicDPDA < > & > ( Rename::rename, "automaton" ).setDocumentation (
"Rename automaton's states and pushdown store symbols.\n\
\n\
@param pda pushdown automaton to rename\n\
@return @p pda with renamed states and pushdown store symbols" );

} /* namespace simplify */

} /* namespace automaton */
