/*
 * Trim.cpp
 *
 *  Created on: 23. 3. 2014
 *	  Author: Tomas Pecka
 */

#include "Trim.h"

#include <automaton/FSM/ExtendedNFA.h>
#include <automaton/FSM/CompactNFA.h>
#include <automaton/FSM/EpsilonNFA.h>
#include <automaton/FSM/NFA.h>
#include <automaton/FSM/DFA.h>
#include <automaton/TA/DFTA.h>
#include <registration/AlgoRegistration.hpp>

namespace automaton {

namespace simplify {

auto TrimDFA = registration::AbstractRegister < Trim, automaton::DFA < >, const automaton::DFA < > & > ( Trim::trim, "fsm" ).setDocumentation (
"Removes inaccessible and useless states from the given automaton. Uses first the @sa UselessStatesRemover and next @sa UnreachableStatesRemover in the process.\n\
\n\
@param fsm finite automaton or finite tree automaton to trim\n\
@return the trimmed automaton equivalent to @p fsm" );

auto TrimNFA = registration::AbstractRegister < Trim, automaton::NFA < >, const automaton::NFA < > & > ( Trim::trim, "fsm" ).setDocumentation (
"Removes inaccessible and useless states from the given automaton. Uses first the @sa UselessStatesRemover and next @sa UnreachableStatesRemover in the process.\n\
\n\
@param fsm finite automaton or finite tree automaton to trim\n\
@return the trimmed automaton equivalent to @p fsm" );

auto TrimMultiInitialStateNFA = registration::AbstractRegister < Trim, automaton::MultiInitialStateNFA < >, const automaton::MultiInitialStateNFA < > & > ( Trim::trim, "fsm" ).setDocumentation (
"Removes inaccessible and useless states from the given automaton. Uses first the @sa UselessStatesRemover and next @sa UnreachableStatesRemover in the process.\n\
\n\
@param fsm finite automaton or finite tree automaton to trim\n\
@return the trimmed automaton equivalent to @p fsm" );

auto TrimEpsilonNFA = registration::AbstractRegister < Trim, automaton::EpsilonNFA < >, const automaton::EpsilonNFA < > & > ( Trim::trim, "fsm" ).setDocumentation (
"Removes inaccessible and useless states from the given automaton. Uses first the @sa UselessStatesRemover and next @sa UnreachableStatesRemover in the process.\n\
\n\
@param fsm finite automaton or finite tree automaton to trim\n\
@return the trimmed automaton equivalent to @p fsm" );

auto TrimCompactNFA = registration::AbstractRegister < Trim, automaton::CompactNFA < >, const automaton::CompactNFA < > & > ( Trim::trim, "fsm" ).setDocumentation (
"Removes inaccessible and useless states from the given automaton. Uses first the @sa UselessStatesRemover and next @sa UnreachableStatesRemover in the process.\n\
\n\
@param fsm finite automaton or finite tree automaton to trim\n\
@return the trimmed automaton equivalent to @p fsm" );

auto TrimExtendedNFA = registration::AbstractRegister < Trim, automaton::ExtendedNFA < >, const automaton::ExtendedNFA < > & > ( Trim::trim, "fsm" ).setDocumentation (
"Removes inaccessible and useless states from the given automaton. Uses first the @sa UselessStatesRemover and next @sa UnreachableStatesRemover in the process.\n\
\n\
@param fsm finite automaton or finite tree automaton to trim\n\
@return the trimmed automaton equivalent to @p fsm" );

auto TrimDFTA = registration::AbstractRegister < Trim, automaton::DFTA < >, const automaton::DFTA < > & > ( Trim::trim, "fsm" ).setDocumentation (
"Removes inaccessible and useless states from the given automaton. Uses first the @sa UselessStatesRemover and next @sa UnreachableStatesRemover in the process.\n\
\n\
@param fsm finite automaton or finite tree automaton to trim\n\
@return the trimmed automaton equivalent to @p fsm" );

} /* namespace simplify */

} /* namespace automaton */

