/*
 * EpsilonRemover.cpp
 *
 *  Created on: 24. 11. 2014
 *	  Author: Jan Travnicek
 */

#include "EpsilonRemover.h"
#include <registration/AlgoRegistration.hpp>

namespace grammar {

namespace simplify {

auto EpsilonRemoverCFG = registration::AbstractRegister < EpsilonRemover, grammar::EpsilonFreeCFG < >, const grammar::CFG < > & > ( EpsilonRemover::remove, "grammar" ).setDocumentation (
"Removes epsilon rules from the given grammar.\n\
\n\
@param grammar the modified grammar\n\
@return grammar without epsilon rules" );

auto EpsilonRemoverEpsilonFreeCFG = registration::AbstractRegister < EpsilonRemover, grammar::EpsilonFreeCFG < >, const grammar::EpsilonFreeCFG < > & > ( EpsilonRemover::remove, "grammar" ).setDocumentation (
"Removes epsilon rules from the given grammar.\n\
\n\
@param grammar the modified grammar\n\
@return grammar without epsilon rules" );

auto EpsilonRemoverCNF = registration::AbstractRegister < EpsilonRemover, grammar::CNF < >, const grammar::CNF < > & > ( EpsilonRemover::remove, "grammar" ).setDocumentation (
"Removes epsilon rules from the given grammar.\n\
\n\
@param grammar the modified grammar\n\
@return grammar without epsilon rules" );

auto EpsilonRemoverGNF = registration::AbstractRegister < EpsilonRemover, grammar::GNF < >, const grammar::GNF < > & > ( EpsilonRemover::remove, "grammar" ).setDocumentation (
"Removes epsilon rules from the given grammar.\n\
\n\
@param grammar the modified grammar\n\
@return grammar without epsilon rules" );

auto EpsilonRemoverLG = registration::AbstractRegister < EpsilonRemover, grammar::EpsilonFreeCFG < >, const grammar::LG < > & > ( EpsilonRemover::remove, "grammar" ).setDocumentation (
"Removes epsilon rules from the given grammar.\n\
\n\
@param grammar the modified grammar\n\
@return grammar without epsilon rules" );

auto EpsilonRemoverLeftLG = registration::AbstractRegister < EpsilonRemover, grammar::EpsilonFreeCFG < >, const grammar::LeftLG < > & > ( EpsilonRemover::remove, "grammar" ).setDocumentation (
"Removes epsilon rules from the given grammar.\n\
\n\
@param grammar the modified grammar\n\
@return grammar without epsilon rules" );

auto EpsilonRemoverLeftRG = registration::AbstractRegister < EpsilonRemover, grammar::LeftRG < >, const grammar::LeftRG < > & > ( EpsilonRemover::remove, "grammar" ).setDocumentation (
"Removes epsilon rules from the given grammar.\n\
\n\
@param grammar the modified grammar\n\
@return grammar without epsilon rules" );

auto EpsilonRemoverRightLG = registration::AbstractRegister < EpsilonRemover, grammar::EpsilonFreeCFG < >, const grammar::RightLG < > & > ( EpsilonRemover::remove, "grammar" ).setDocumentation (
"Removes epsilon rules from the given grammar.\n\
\n\
@param grammar the modified grammar\n\
@return grammar without epsilon rules" );

auto EpsilonRemoverRightRG = registration::AbstractRegister < EpsilonRemover, grammar::RightRG < >, const grammar::RightRG < > & > ( EpsilonRemover::remove, "grammar" ).setDocumentation (
"Removes epsilon rules from the given grammar.\n\
\n\
@param grammar the modified grammar\n\
@return grammar without epsilon rules" );

} /* namespace simplify */

} /* namespace grammar */
