/*
 * ExactCompare.cpp
 *
 *  Created on: Oct 9, 2014
 *	  Author: Radomir Polach */

#include "ExactCompare.h"
#include <registration/AlgoRegistration.hpp>

namespace string {

namespace naive {

auto ExactCompareEpsilon = registration::AbstractRegister < ExactCompare, int, const string::Epsilon < > &, const string::Epsilon < > & > ( ExactCompare::compare, "u", "v" ).setDocumentation (
"Implementation of exact comparison of strings.\n\
\n\
@param u the first string to compare\n\
@param v the second string to compare\n\
@return zero as epsilons are equal" );

auto ExactCompareLinearString = registration::AbstractRegister < ExactCompare, int, const string::LinearString < > &, const string::LinearString < > & > ( ExactCompare::compare, "u", "v" ).setDocumentation (
"Implementation of exact comparison of strings.\n\
\n\
@param u the first string to compare\n\
@param v the second string to compare\n\
@return negative value if u compares smaller than v, positive value if u compares bigger than v, zero if u and v are equal" );

auto ExactCompareCyclicString = registration::AbstractRegister < ExactCompare, int, const string::CyclicString < > &, const string::CyclicString < > & > ( ExactCompare::compare, "u", "v" ).setDocumentation (
"Implementation of exact comparison of strings without taking rotation into account.\n\
\n\
@param u the first string to compare\n\
@param v the second string to compare\n\
@return negative value if u compares smaller than v, positive value if u compares bigger than v, zero if u and v are equal" );

} /* namespace naive */

} /* namespace string */
