/*
 * Author: Radovan Cerveny
 */

#ifndef NAIVE_DAWG_MATCHER_CONSTRUCTION_H_
#define NAIVE_DAWG_MATCHER_CONSTRUCTION_H_

#include <automaton/FSM/DFA.h>
#include <string/LinearString.h>

#include <automaton/determinize/Determinize.h>
#include <automaton/simplify/Minimize.h>
#include <automaton/simplify/EpsilonRemoverIncoming.h>
#include <stringology/indexing/NondeterministicExactSuffixAutomaton.h>

namespace stringology {

namespace matching {

class NaiveDAWGMatcherConstruction {
public:
	/**
	 * Naive construction of minimal suffix automaton for given pattern - EpsNFA -> NFA -> DFA -> minDFA -> removeErrorState.
	 * @return minimal suffix automaton for given pattern.
	 */
	template < class SymbolType >
	static automaton::DFA < SymbolType, ext::set < unsigned > > naiveConstruct ( const string::LinearString < SymbolType > & pattern );

};

template < class SymbolType >
automaton::DFA < SymbolType, ext::set < unsigned > > NaiveDAWGMatcherConstruction::naiveConstruct ( const string::LinearString < SymbolType > & pattern ) {
	auto patternData = pattern.getContent ( );
	reverse ( patternData.begin ( ), patternData.end ( ) );
	string::LinearString < SymbolType > reversedPattern ( pattern.getAlphabet ( ), std::move ( patternData ) );

	automaton::EpsilonNFA < SymbolType, DefaultEpsilonType, unsigned > nfaSuffixAutomaton = stringology::indexing::NondeterministicExactSuffixAutomaton::construct ( reversedPattern );

	return automaton::simplify::Minimize::minimize ( automaton::determinize::Determinize::determinize ( automaton::simplify::EpsilonRemoverIncoming::remove ( nfaSuffixAutomaton ) ) );

}

} /* namespace matching */

} /* namespace stringology */

#endif /* NAIVE_DAWG_MATCHER_CONSTRUCTION_H_ */
