#include <Utils.hpp>
#include <abstraction/ImmediateValueAbstraction.hpp>
#include <common/AlgorithmHelper.h>

namespace Utils
{
    QRectF pointsToRect(QPointF a, QPointF b) {
        qreal minx = std::min(a.x(), b.x());
        qreal maxx = std::max(a.x(), b.x());
        qreal miny = std::min(a.y(), b.y());
        qreal maxy = std::max(a.y(), b.y());
        return {{minx, miny}, QPointF {maxx, maxy}};
    }

    std::shared_ptr<abstraction::OperationAbstraction> generateRandomAutomaton() {
        ext::vector<std::shared_ptr<abstraction::OperationAbstraction>> params;
        params.push_back(std::make_shared<abstraction::ImmediateValueAbstraction<size_t>>(5));
        params.push_back(std::make_shared<abstraction::ImmediateValueAbstraction<size_t>>(3));
        params.push_back(std::make_shared<abstraction::ImmediateValueAbstraction<bool>>(true));
        params.push_back(std::make_shared<abstraction::ImmediateValueAbstraction<double>>(10.0));

        std::vector<bool> moves(params.size(), true);

        return abstraction::AlgorithmHelper::eval("automaton::generate::RandomAutomatonFactory",
                                                  {},
                                                  params,
                                                  moves,
                                                  abstraction::AlgorithmCategories::AlgorithmCategory::DEFAULT);
    }

    std::shared_ptr<abstraction::OperationAbstraction> generateRandomGrammar() {
        ext::vector<std::shared_ptr<abstraction::OperationAbstraction>> params;
        params.push_back(std::make_shared<abstraction::ImmediateValueAbstraction<size_t>>(5));
        params.push_back(std::make_shared<abstraction::ImmediateValueAbstraction<size_t>>(3));
        params.push_back(std::make_shared<abstraction::ImmediateValueAbstraction<bool>>(true));
        params.push_back(std::make_shared<abstraction::ImmediateValueAbstraction<double>>(30.0));

        std::vector<bool> moves(params.size(), true);

        return abstraction::AlgorithmHelper::eval("grammar::generate::RandomGrammarFactory",
                                                  {},
                                                  params,
                                                  moves,
                                                  abstraction::AlgorithmCategories::AlgorithmCategory::DEFAULT);
    }
}
