#include "TestFiles.hpp"
#include "testfiles_basedir.hpp"

#include <exception/CommonException.h>
#include <experimental/filesystem>

std::vector < std::string > TestFiles::Get ( const std::string& regex ) {
	return Get ( std::regex ( regex ) );
}

std::vector < std::string > TestFiles::Get ( const std::regex& regex ) {
	std::vector < std::string > res;

	try {
		for ( const std::experimental::filesystem::directory_entry & entry: std::experimental::filesystem::recursive_directory_iterator ( TEST_FILES_BASEDIR ) ) {
			std::smatch match;
			const std::string path = entry.path ( );

			if ( std::regex_search ( path, match, regex ) ) {
				res.push_back ( path );
			}
		}
	} catch ( const std::experimental::filesystem::filesystem_error & e ) {
		throw exception::CommonException ( e.what ( ) );
	}

	return res;
}

std::string TestFiles::GetOne ( const std::string& regex ) {
	return GetOne ( std::regex ( regex ) );
}

std::string TestFiles::GetOne ( const std::regex& regex ) {
	std::vector < std::string > res = Get ( regex );

	if ( res.size ( ) > 1 )
		throw exception::CommonException ( std::string ( "Multiple files found for TestFiles::GetOne. TestFilesDir is " ) + TEST_FILES_BASEDIR );

	if ( res.size ( ) == 0 )
		throw exception::CommonException ( std::string ( "No files found for TestFiles::GetOne. TestFilesDir is " ) + TEST_FILES_BASEDIR );

	return res.at ( 0 );
}
