/*
 * FormalRTE.h
 *
 * Created on: Sep 27, 2017
 * Author: Jan Travnicek
 */

#ifndef _STRING_FORMAL_RTE_H_
#define _STRING_FORMAL_RTE_H_

#include <alib/memory>
#include <alib/utility>

#include <rte/formal/FormalRTEElements.h>
#include <rte/formal/FormalRTEStructure.h>
#include <rte/formal/FormalRTE.h>
#include <core/stringApi.hpp>

#include <rte/RTEFromStringLexer.h>

namespace core {

template < class SymbolType, class RankType >
struct stringApi < rte::FormalRTEStructure < SymbolType, RankType > > {
	static rte::FormalRTEStructure < SymbolType, RankType > parse ( std::istream & input );
	static bool first ( std::istream & input );
	static void compose ( std::ostream & output, const rte::FormalRTEStructure < SymbolType, RankType > & rte );
private:
	static ext::ptr_value < rte::FormalRTEElement < SymbolType, RankType > > alternation(std::istream & input);
	static ext::ptr_value < rte::FormalRTEElement < SymbolType, RankType > > alternationCont(std::istream & input, ext::ptr_value < rte::FormalRTEElement < SymbolType, RankType > > left);

	static ext::ptr_value < rte::FormalRTEElement < SymbolType, RankType > > substitution(std::istream & input);
	static ext::ptr_value < rte::FormalRTEElement < SymbolType, RankType > > substitutionCont(std::istream & input, ext::ptr_value < rte::FormalRTEElement < SymbolType, RankType > > left);

	static ext::ptr_value < rte::FormalRTEElement < SymbolType, RankType > > factor(std::istream & input);
	static ext::ptr_value < rte::FormalRTEElement < SymbolType, RankType > > star(std::istream & input, rte::FormalRTEElement < SymbolType, RankType > && elem);
	static rte::FormalRTESymbolSubst < SymbolType, RankType > substitutionSymbol ( std::istream & input );
	static ext::ptr_vector < rte::FormalRTEElement < SymbolType, RankType > > arguments ( std::istream & input );

	enum class Priority {
		ALTERNATION,
		CONCATENATION,
		FACTOR
	};

	class Formal {
	public:
		static void visit( const rte::FormalRTEAlternation < SymbolType, RankType > & alternation, ext::tuple < Priority &, std::ostream & > & output);
		static void visit( const rte::FormalRTESubstitution < SymbolType, RankType > & substitution, ext::tuple < Priority &, std::ostream & > & output);
		static void visit( const rte::FormalRTEIteration < SymbolType, RankType > & iteration, ext::tuple < Priority &, std::ostream & > & output);
		static void visit( const rte::FormalRTESymbolAlphabet < SymbolType, RankType > & symbol, ext::tuple < Priority &, std::ostream & > & output);
		static void visit( const rte::FormalRTESymbolSubst < SymbolType, RankType > & epsilon, ext::tuple < Priority &, std::ostream & > & output);
		static void visit( const rte::FormalRTEEmpty < SymbolType, RankType > & empty, ext::tuple < Priority &, std::ostream & > & output);
	};

	class FormalReplace {
	public:
		static std::optional < rte::FormalRTESymbolSubst < SymbolType, RankType > > visit( rte::FormalRTEAlternation < SymbolType, RankType > & alternation, ext::set < common::ranked_symbol < SymbolType, RankType > > & constants);
		static std::optional < rte::FormalRTESymbolSubst < SymbolType, RankType > > visit( rte::FormalRTESubstitution < SymbolType, RankType > & substitution, ext::set < common::ranked_symbol < SymbolType, RankType > > & constants);
		static std::optional < rte::FormalRTESymbolSubst < SymbolType, RankType > > visit( rte::FormalRTEIteration < SymbolType, RankType > & iteration, ext::set < common::ranked_symbol < SymbolType, RankType > > & constants);
		static std::optional < rte::FormalRTESymbolSubst < SymbolType, RankType > > visit( rte::FormalRTESymbolAlphabet < SymbolType, RankType > & symbol, ext::set < common::ranked_symbol < SymbolType, RankType > > & constants);
		static std::optional < rte::FormalRTESymbolSubst < SymbolType, RankType > > visit( rte::FormalRTESymbolSubst < SymbolType, RankType > & epsilon, ext::set < common::ranked_symbol < SymbolType, RankType > > & constants);
		static std::optional < rte::FormalRTESymbolSubst < SymbolType, RankType > > visit( rte::FormalRTEEmpty < SymbolType, RankType > & empty, ext::set < common::ranked_symbol < SymbolType, RankType > > & constants);
	};
};

template < class SymbolType, class RankType >
rte::FormalRTEStructure < SymbolType, RankType > stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::parse ( std::istream & input ) {
	rte::FormalRTEStructure < SymbolType, RankType > res ( alternation ( input ) );
	ext::set < common::ranked_symbol < SymbolType, RankType > > constants = res.getStructure ( ).computeMinimalAlphabets ( ).second;
	rte::FormalRTEElement < SymbolType, RankType > & elem = res.getStructure ( );
	elem.template accept < std::optional < rte::FormalRTESymbolSubst < SymbolType, RankType > >, stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::FormalReplace > ( constants );
	return res;
}

template < class SymbolType, class RankType >
ext::ptr_value < rte::FormalRTEElement < SymbolType, RankType > > stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::alternation(std::istream & input) {
	return alternationCont(input, substitution(input));
}

template < class SymbolType, class RankType >
ext::ptr_value < rte::FormalRTEElement < SymbolType, RankType > > stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::alternationCont(std::istream & input, ext::ptr_value < rte::FormalRTEElement < SymbolType, RankType > > left) {
	rte::RTEFromStringLexer::Token token = rte::RTEFromStringLexer::next(input);
	if(token.type == rte::RTEFromStringLexer::TokenType::PLUS) {
		ext::ptr_value < rte::FormalRTEElement < SymbolType, RankType > > res ( rte::FormalRTEAlternation < SymbolType, RankType > ( std::move ( left ), substitution ( input ) ) );
		return alternationCont ( input, std::move ( res ) );
	} else {
		rte::RTEFromStringLexer::putback(input, token);
		return left;
	}
}

template < class SymbolType, class RankType >
ext::ptr_value < rte::FormalRTEElement < SymbolType, RankType > > stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::substitution(std::istream & input) {
	return substitutionCont(input, factor(input));
}

template < class SymbolType, class RankType >
ext::ptr_value < rte::FormalRTEElement < SymbolType, RankType > > stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::substitutionCont(std::istream & input, ext::ptr_value < rte::FormalRTEElement < SymbolType, RankType > > left) {
	rte::RTEFromStringLexer::Token token = rte::RTEFromStringLexer::next(input);
	if ( token.type == rte::RTEFromStringLexer::TokenType::DOT ) {
		rte::FormalRTESymbolSubst < SymbolType, RankType > symbol = substitutionSymbol ( input );
		ext::ptr_value < rte::FormalRTEElement < SymbolType, RankType > > res ( rte::FormalRTESubstitution < SymbolType, RankType > ( std::move ( left ), factor ( input ), std::move ( symbol ) ) );
		return substitutionCont ( input, std::move ( res ) );
	} else {
		rte::RTEFromStringLexer::putback(input, token);
		return left;
	}
}
template < class SymbolType, class RankType >
rte::FormalRTESymbolSubst < SymbolType, RankType > stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::substitutionSymbol ( std::istream & input ) {
	SymbolType symbol = core::stringApi < SymbolType >::parse ( input );

	rte::RTEFromStringLexer::Token token = rte::RTEFromStringLexer::next ( input );
	if ( token.type != rte::RTEFromStringLexer::TokenType::RANK )
		throw exception::CommonException ( "Missing rank." );

	unsigned rank = ext::from_string < unsigned > ( token.value );

	if ( rank != 0 )
		throw exception::CommonException ( "Substitution symbol must have zero rank." );

	return rte::FormalRTESymbolSubst < SymbolType, RankType > ( common::ranked_symbol < SymbolType, RankType > ( std::move ( symbol ), RankType ( rank ) ) );
}

template < class SymbolType, class RankType >
ext::ptr_value < rte::FormalRTEElement < SymbolType, RankType > > stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::factor(std::istream & input) {
	rte::RTEFromStringLexer::Token token = rte::RTEFromStringLexer::next(input);
	if(token.type == rte::RTEFromStringLexer::TokenType::LPAR) {
		ext::ptr_value < rte::FormalRTEElement < SymbolType, RankType > > base = alternation(input);
		token = rte::RTEFromStringLexer::next(input);
		if(token.type != rte::RTEFromStringLexer::TokenType::RPAR) throw exception::CommonException("Expected RPAR");
		return star(input, std::move ( base ) );
	} else if(token.type == rte::RTEFromStringLexer::TokenType::EMPTY) {
		return star(input, ext::ptr_value < rte::FormalRTEElement < SymbolType, RankType > > ( rte::FormalRTEEmpty < SymbolType, RankType > ( ) ) );
	} else if(token.type == rte::RTEFromStringLexer::TokenType::ERROR) {
		rte::RTEFromStringLexer::putback(input, token);

		SymbolType symbol = core::stringApi < SymbolType >::parse ( input );

		token = rte::RTEFromStringLexer::next ( input );
		if ( token.type != rte::RTEFromStringLexer::TokenType::RANK )
			throw exception::CommonException ( "Missing rank." );

		unsigned rank = ext::from_string < unsigned > ( token.value );

		ext::ptr_vector < rte::FormalRTEElement < SymbolType, RankType > > children = arguments ( input );

		rte::FormalRTESymbolAlphabet < SymbolType, RankType > res ( common::ranked_symbol < SymbolType, RankType > ( std::move ( symbol ), rank ), children );
		return star ( input, std::move ( res ) );
	} else {
		throw exception::CommonException("Unrecognised token at factor rule");
	}
}

template < class SymbolType, class RankType >
ext::ptr_value < rte::FormalRTEElement < SymbolType, RankType > > stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::star(std::istream & input, rte::FormalRTEElement < SymbolType, RankType > && elem) {
	rte::RTEFromStringLexer::Token token = rte::RTEFromStringLexer::next(input);
	if(token.type == rte::RTEFromStringLexer::TokenType::STAR) {
		rte::FormalRTESymbolSubst < SymbolType, RankType > symbol = substitutionSymbol ( input );
		rte::FormalRTEIteration < SymbolType, RankType > iter ( std::move ( elem ), std::move ( symbol ) );
		return star ( input, std::move ( iter ) );
	} else {
		rte::RTEFromStringLexer::putback(input, token);
		return ext::ptr_value < rte::FormalRTEElement < SymbolType, RankType > > ( std::move ( elem ) );
	}
}

template < class SymbolType, class RankType >
ext::ptr_vector < rte::FormalRTEElement < SymbolType, RankType > > stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::arguments ( std::istream & input ) {
	rte::RTEFromStringLexer::Token token = rte::RTEFromStringLexer::next(input);
	if ( token.type != rte::RTEFromStringLexer::TokenType::LPAR ) {
		rte::RTEFromStringLexer::putback ( input, token );
		return { };
	}

	token = rte::RTEFromStringLexer::next(input);
	if ( token.type == rte::RTEFromStringLexer::TokenType::RPAR )
		return { };

	ext::ptr_vector < rte::FormalRTEElement < SymbolType, RankType > > res;
	rte::RTEFromStringLexer::putback ( input, token );
	do {
		res.push_back ( ( rte::FormalRTEElement < SymbolType, RankType > && ) alternation ( input ) );
		token = rte::RTEFromStringLexer::next ( input );
	} while ( token.type == rte::RTEFromStringLexer::TokenType::COMMA );

	if ( token.type != rte::RTEFromStringLexer::TokenType::RPAR )
		throw exception::CommonException("Expected RPAR");

	return res;
}

template < class SymbolType, class RankType >
std::optional < rte::FormalRTESymbolSubst < SymbolType, RankType > > stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::FormalReplace::visit( rte::FormalRTEAlternation < SymbolType, RankType > & alternation, ext::set < common::ranked_symbol < SymbolType, RankType > > & constants) {
	std::optional < rte::FormalRTESymbolSubst < SymbolType, RankType > > left = alternation.getLeftElement ( ).template accept < std::optional < rte::FormalRTESymbolSubst < SymbolType, RankType > >, stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::FormalReplace > ( constants );
	if ( left )
		alternation.setLeftElement ( std::move ( left.value ( ) ) );

	std::optional < rte::FormalRTESymbolSubst < SymbolType, RankType > > right = alternation.getRightElement ( ).template accept < std::optional < rte::FormalRTESymbolSubst < SymbolType, RankType > >, stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::FormalReplace > ( constants );
	if ( right )
		alternation.setRightElement ( std::move ( right.value ( ) ) );

	return std::nullopt;
}

template < class SymbolType, class RankType >
std::optional < rte::FormalRTESymbolSubst < SymbolType, RankType > > stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::FormalReplace::visit( rte::FormalRTESubstitution < SymbolType, RankType > & substitution, ext::set < common::ranked_symbol < SymbolType, RankType > > & constants) {
	std::optional < rte::FormalRTESymbolSubst < SymbolType, RankType > > left = substitution.getLeftElement ( ).template accept < std::optional < rte::FormalRTESymbolSubst < SymbolType, RankType > >, stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::FormalReplace > ( constants );
	if ( left )
		substitution.setLeftElement ( std::move ( left.value ( ) ) );

	std::optional < rte::FormalRTESymbolSubst < SymbolType, RankType > > right = substitution.getRightElement ( ).template accept < std::optional < rte::FormalRTESymbolSubst < SymbolType, RankType > >, stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::FormalReplace > ( constants );
	if ( right )
		substitution.setRightElement ( std::move ( right.value ( ) ) );

	return std::nullopt;
}

template < class SymbolType, class RankType >
std::optional < rte::FormalRTESymbolSubst < SymbolType, RankType > > stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::FormalReplace::visit( rte::FormalRTEIteration < SymbolType, RankType > & iteration, ext::set < common::ranked_symbol < SymbolType, RankType > > & constants) {
	std::optional < rte::FormalRTESymbolSubst < SymbolType, RankType > > element = iteration.getElement ( ).template accept < std::optional < rte::FormalRTESymbolSubst < SymbolType, RankType > >, stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::FormalReplace > ( constants );
	if ( element )
		iteration.setElement ( std::move ( element.value ( ) ) );

	return std::nullopt;
}

template < class SymbolType, class RankType >
std::optional < rte::FormalRTESymbolSubst < SymbolType, RankType > > stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::FormalReplace::visit( rte::FormalRTESymbolAlphabet < SymbolType, RankType > & symbol, ext::set < common::ranked_symbol < SymbolType, RankType > > & constants) {
	if ( constants.count ( symbol.getSymbol ( ) ) )
		return rte::FormalRTESymbolSubst < SymbolType, RankType > ( symbol.getSymbol ( ) );

	for ( unsigned i = 0; i < symbol.getElements ( ).size ( ); ++ i ) {
		rte::FormalRTEElement < SymbolType, RankType > & elem = symbol.getElement ( i );
		std::optional < rte::FormalRTESymbolSubst < SymbolType, RankType > > res = elem.template accept < std::optional < rte::FormalRTESymbolSubst < SymbolType, RankType > >, stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::FormalReplace > ( constants );
		if ( res )
			symbol.setElement ( i, std::move ( res.value ( ) ) );
	}

	return std::nullopt;
}

template < class SymbolType, class RankType >
std::optional < rte::FormalRTESymbolSubst < SymbolType, RankType > > stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::FormalReplace::visit( rte::FormalRTESymbolSubst < SymbolType, RankType > &, ext::set < common::ranked_symbol < SymbolType, RankType > > & ) {
	return std::nullopt;
}

template < class SymbolType, class RankType >
std::optional < rte::FormalRTESymbolSubst < SymbolType, RankType > > stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::FormalReplace::visit( rte::FormalRTEEmpty < SymbolType, RankType > &, ext::set < common::ranked_symbol < SymbolType, RankType > > & ) {
	return std::nullopt;
}

template < class SymbolType, class RankType >
bool stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::first ( std::istream & ) {
	return true;
}

template < class SymbolType, class RankType >
void stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::compose ( std::ostream & output, const rte::FormalRTEStructure < SymbolType, RankType > & rte ) {
	Priority tmp = Priority::ALTERNATION;
	ext::tuple < Priority &, std::ostream & > out = ext::tie ( tmp, output );
	rte.getStructure ( ).template accept < void, stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::Formal > ( out );
}

template < class SymbolType, class RankType >
void stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::Formal::visit( const rte::FormalRTEAlternation < SymbolType, RankType > & alternation, ext::tuple < Priority &, std::ostream & > & out ) {
	Priority outerPriorityMinimum = std::get < 0 > ( out );
	if ( outerPriorityMinimum == Priority::CONCATENATION || outerPriorityMinimum == Priority::FACTOR ) std::get < 1 > ( out ) << '(';

	std::get < 0 > ( out ) = Priority::ALTERNATION;
	alternation.getLeftElement ( ).template accept < void, stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::Formal > ( out );
	std::get < 1 > ( out ) << '+';
	alternation.getRightElement ( ).template accept < void, stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::Formal > ( out );

	if ( outerPriorityMinimum == Priority::CONCATENATION || outerPriorityMinimum == Priority::FACTOR ) std::get < 1 > ( out ) << ')';
}

template < class SymbolType, class RankType >
void stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::Formal::visit( const rte::FormalRTESubstitution < SymbolType, RankType > & substitution, ext::tuple < Priority &, std::ostream & > & out ) {
	Priority outerPriorityMinimum = std::get < 0 > ( out );
	if ( outerPriorityMinimum == Priority::FACTOR ) std::get < 1 > ( out ) << '(';

	std::get < 0 > ( out ) = Priority::CONCATENATION;
	substitution.getLeftElement ( ).template accept < void, stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::Formal > ( out );
	std::get < 1 > ( out ) << '.';
	stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::Formal::visit ( substitution.getSubstitutionSymbol ( ), out );
	std::get < 1 > ( out ) << ' ';
	std::get < 0 > ( out ) = Priority::CONCATENATION;
	substitution.getRightElement ( ).template accept < void, stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::Formal > ( out );

	if ( outerPriorityMinimum == Priority::FACTOR ) std::get < 1 > ( out ) << ')';
}

template < class SymbolType, class RankType >
void stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::Formal::visit( const rte::FormalRTEIteration < SymbolType, RankType > & iteration, ext::tuple < Priority &, std::ostream & > & out ) {
	std::get < 0 > ( out ) = Priority::FACTOR;
	iteration.getElement ( ).template accept < void, stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::Formal > ( out );
	std::get < 1 > ( out ) << "*";
	stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::Formal::visit ( iteration.getSubstitutionSymbol ( ), out );
}

template < class SymbolType, class RankType >
void stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::Formal::visit( const rte::FormalRTESymbolAlphabet < SymbolType, RankType > & symbol, ext::tuple < Priority &, std::ostream & > & out ) {
	core::stringApi < SymbolType >::compose ( std::get < 1 > ( out ), symbol.getSymbol ( ).getSymbol ( ) );

	std::get < 1 > ( out ) << " " << ext::to_string ( symbol.getSymbol ( ).getRank ( ) );

	if ( symbol.getElements ( ).empty ( ) )
		return;

	std::get < 1 > ( out ) << " ( ";

	bool first = true;
	for ( const rte::FormalRTEElement < SymbolType, RankType > & c : symbol.getElements ( ) ) {
		if ( first )
			first = false;
		else
			std::get < 1 > ( out ) << ", ";

		std::get < 0 > ( out ) = Priority::ALTERNATION;
		c.template accept < void, stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::Formal > ( out );
	}

	std::get < 1 > ( out ) << ")";
}

template < class SymbolType, class RankType >
void stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::Formal::visit( const rte::FormalRTESymbolSubst < SymbolType, RankType > & symbol, ext::tuple < Priority &, std::ostream & > & out ) {
	core::stringApi < SymbolType >::compose ( std::get < 1 > ( out ), symbol.getSymbol ( ).getSymbol ( ) );

	std::get < 1 > ( out ) << " " << ext::to_string ( symbol.getSymbol ( ).getRank ( ) );
}

template < class SymbolType, class RankType >
void stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::Formal::visit( const rte::FormalRTEEmpty < SymbolType, RankType > &, ext::tuple < Priority &, std::ostream & > & out ) {
	std::get < 1 > ( out ) << "#0";
}

template < class SymbolType, class RankType >
struct stringApi < rte::FormalRTE < SymbolType, RankType > > {
	static rte::FormalRTE < SymbolType, RankType > parse ( std::istream & input );
	static bool first ( std::istream & input );
	static void compose ( std::ostream & output, const rte::FormalRTE < SymbolType, RankType > & rte );
};

template < class SymbolType, class RankType >
rte::FormalRTE < SymbolType, RankType > stringApi < rte::FormalRTE < SymbolType, RankType > >::parse ( std::istream & input ) {
	return rte::FormalRTE < SymbolType, RankType > ( core::stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::parse ( input ) );
}

template < class SymbolType, class RankType >
bool stringApi < rte::FormalRTE < SymbolType, RankType > >::first ( std::istream & input ) {
	return core::stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::first ( input );
}

template < class SymbolType, class RankType >
void stringApi < rte::FormalRTE < SymbolType, RankType > >::compose ( std::ostream & output, const rte::FormalRTE < SymbolType, RankType > & rte ) {
	core::stringApi < rte::FormalRTEStructure < SymbolType, RankType > >::compose ( output, rte.getRTE ( ) );
}

} /* namespace core */

#endif /* _STRING_FORMAL_RTE_H_ */
