/*
 * XmlContainerParserRegistry.cpp
 *
 *  Created on: 19. 8. 2017
 *	  Author: Jan Travnicek
 */

#include <registry/XmlContainerParserRegistry.hpp>
#include <alib/foreach>

namespace abstraction {

ext::map < std::string, ext::list < ext::pair < std::string, std::shared_ptr < XmlContainerParserRegistry::Entry > > > > & XmlContainerParserRegistry::getEntries ( ) {
	static ext::map < std::string, ext::list < ext::pair < std::string, std::shared_ptr < Entry > > > > containerGroups;
	return containerGroups;
}

bool XmlContainerParserRegistry::hasAbstraction ( const std::string & container ) {
	return getEntries ( ).count ( container );
}

std::shared_ptr < abstraction::OperationAbstraction > XmlContainerParserRegistry::getAbstraction ( const std::string & container, const std::string & type ) {
	auto group = getEntries ( ).find ( container );
	if ( group == getEntries ( ).end ( ) )
		throw exception::CommonException ( "Entry " + container + " not available" );

	for ( const ext::pair < std::string, std::shared_ptr < Entry > > & entry : group->second )
		if ( ext::is_same_type ( type, ext::erase_template_info ( entry.first ) ) )
			return entry.second->getAbstraction ( );

	throw exception::CommonException ( "Entry for " + container + " parametrized with " + type + " not available." );
}

ext::set < std::string > XmlContainerParserRegistry::listOverloads ( const std::string & container ) {
	auto group = getEntries ( ).find ( container );
	if ( group == getEntries ( ).end ( ) )
		throw exception::CommonException ( "Entry " + container + " not available" );

	ext::set < std::string > res;
	for ( const ext::pair < std::string, std::shared_ptr < Entry > > & entry : group->second )
		res.insert ( entry.first );

	return res;
}

ext::set < std::string > XmlContainerParserRegistry::list ( ) {
	ext::set < std::string > res;

	for ( const std::pair < const std::string, ext::list < ext::pair < std::string, std::shared_ptr < Entry > > > > & groups : getEntries ( ) )
		res.insert ( groups.first );

	return res;
}

} /* namespace abstraction */
