/*
 * ReadlinePromptCompletion.cpp
 *
 *  Created on: 22. 1. 2019
 *	  Author: Tomas Pecka
 */

#include <alib/registration>

#include "ReadlinePromptCompletion.h"

static ext::Register < void > instance ( [ ] ( ) {
		// register readline completion function, pass environment
		rl_attempted_completion_function = ReadlinePromptCompletion::readline_completion;
	} );

/* ========================================================================= */

std::set < std::string > ReadlinePromptCompletion::addPrefix ( const std::set < std::string > & collection, const std::string & prefix ) {
	std::set < std::string > res;
	for ( const std::string & s : collection )
		res.insert ( prefix + s );

	return res;
}

bool ReadlinePromptCompletion::startswith ( const std::string & subject, const std::string & prefix ) {
	if ( prefix.size ( ) > subject.size ( ) )
		return false;

	return subject.substr ( 0, prefix.size ( ) ) == prefix;
}

std::set < std::string > ReadlinePromptCompletion::filter_completions ( const std::set < std::string > & choices, const char *text ) {
	std::pair < std::set < std::string > :: const_iterator, std::set < std::string > :: const_iterator > range;
	const std::string prefix = text;

	range = std::equal_range ( choices.begin ( ), choices.end ( ), prefix,
			[ &prefix ] ( const std::string & a, const std::string & b ) { return strncmp ( a.c_str ( ), b.c_str ( ), prefix.size ( )) < 0; } );

	return std::set < std::string > ( range.first, range.second );
}

/* ========================================================================= */

ReadlinePromptCompletion::CompletionContext ReadlinePromptCompletion::context ( const char *text, const int start, const int end ) {
	std::string line ( rl_line_buffer );

	if ( start == 0 )
		return CompletionContext::COMMAND;

	if ( startswith ( line, "introspect" ) && start == 11 ) {
		return CompletionContext::COMMAND_INTROSPECT;
	}
	else if ( startswith ( line, "introspect overloads" ) && start == 21 )
		return CompletionContext::ALGORITHM;

	else if ( startswith ( line, "introspect variables" ) && start == 21 )
		return CompletionContext::VARIABLE;

	else if ( startswith ( line, "introspect bindings" ) && start == 20 )
		return CompletionContext::BINDING;

	/* TODO
	else if ( startswith ( line, "introspect algorithms" ) && start == 21 )
	else if ( startswith ( line, "introspect casts" ) && start == 17 )
	else if ( startswith ( line, "introspect datatypes" ) && start == 20 )
	*/

	if ( end - start > 0 && text [ 0 ] == '$' )
		return CompletionContext::VARIABLE;
	else if ( end - start > 0 && text [ 0 ] == '#' )
		return CompletionContext::BINDING;

	/* scan the context backwards. If first non-whitespace character is < or >, then complete a filename */
	char *p = rl_line_buffer + start - 1;
	while ( p >= rl_line_buffer && isspace ( *p ) ) p --;
	if ( p >= rl_line_buffer && ( *p == '<' || *p == '>' ) )
		return CompletionContext::FILEPATH_OR_VARIABLE;

	if ( p >= rl_line_buffer && ( *p == '|' || *p == '(' ) )
		return CompletionContext::ALGORITHM;

	if ( startswith ( line, "execute" ) && start == 8 )
		return CompletionContext::ALGORITHM;

	/* undecided, fallback to filepath */
	return CompletionContext::FILEPATH;
}

/* ========================================================================= */

char ** ReadlinePromptCompletion::readline_completion ( const char *text, int start, int end ) {
	// for variables and bindings
	rl_special_prefixes = "$#@";
	rl_attempted_completion_over = 1;

	// std::cerr << ">" << text << "< " << start << " " << end << std::endl;

	switch ( context ( text, start, end ) ) {
		case CompletionContext::ALGORITHM:
			return rl_completion_matches ( text, complete_algorithm );
		case CompletionContext::COMMAND:
			return rl_completion_matches ( text, complete_command );
		case CompletionContext::COMMAND_INTROSPECT:
			return rl_completion_matches ( text, complete_command_introspect );
		case CompletionContext::VARIABLE:
			return rl_completion_matches ( text, complete_variable );
		case CompletionContext::BINDING:
			return rl_completion_matches ( text, complete_binding );
		case CompletionContext::FILEPATH:
			return rl_completion_matches ( text, complete_filepath );
		case CompletionContext::FILEPATH_OR_VARIABLE:
			return rl_completion_matches ( text, complete_filepath_or_variable );
		default:
			return nullptr;
	}
}
