/*
 * ReadlinePromptCompletion.h
 *
 *  Created on: 26. 1. 2019
 *	  Author: Jan Travnicek
 */

#ifndef _READLINE_PROMPT_HISTORY_H
#define _READLINE_PROMPT_HISTORY_H

#include <string.h>
#include <stdio.h>

#include <alib/string>

#include <readline/history.h>

class ReadlinePromptHistory {
	static char esc_char [];
	static char essc_str [];
	static size_t esc_char_size;

	static char * descape ( const char * buffer ) {
		size_t l = strlen ( buffer );

		char * dest = ( char * ) malloc ( ( l + 1 ) * sizeof ( char ) );
		char * ptr = dest;
		for ( size_t i = 0; i < l; ++ i ) {
			if ( buffer [ i ] == '\\' ) {
				++ i;
				size_t j = std::find ( essc_str, essc_str + esc_char_size, buffer [ i ] ) - essc_str;
				if ( j == esc_char_size ) {
					free ( dest );
					return strdup ( buffer );
				}
				* ptr ++ = esc_char [ j ];
			} else {
				* ptr ++ = buffer [ i ];
			}
		}
		* ptr = '\0';

		return dest;
	}

	static char * escape ( const char * buffer){
		size_t l = strlen ( buffer );

		char * dest = ( char * ) malloc ( ( l * 2 + 1 ) * sizeof ( char ) );
		char * ptr = dest;
		for ( size_t i = 0; i < l; ++ i ) {
			size_t j = std::find ( esc_char, esc_char + esc_char_size, buffer [ i ] ) - esc_char;
			if ( j == esc_char_size ) {
				* ptr ++ = buffer [ i ];
			} else {
				* ptr ++ = '\\';
				* ptr ++ = essc_str [ j ];
			}
		}
		* ptr = '\0';

		return dest;
	}

	template < class Callable >
	static void history_transform ( Callable callable ) {
		HIST_ENTRY ** history = history_list ( );
		int i = 0;
		if ( history ) while ( * history ) {
			char * tmp = callable ( ( * history )->line );
			replace_history_entry ( i, tmp, ( * history )->data );
			free ( tmp );
			++ history;
			++ i;
		}
	}

public:
	static void readHistory ( const std::string & history_file ) {
		read_history ( history_file.c_str ( ) );
		ReadlinePromptHistory::history_transform ( ReadlinePromptHistory::descape );
	}

	static void writeHistory ( const std::string & history_file ) {
		ReadlinePromptHistory::history_transform ( ReadlinePromptHistory::escape );
		write_history ( history_file.c_str ( ) );
	}

	static void addHistory ( const std::string & line ) {
		add_history ( line.c_str ( ) );
	}

};

#endif /* _READLINE_PROMPT_HISTORY_H */
